/*
 * usage:  erode <b|d> <size> <inputfile>
 *
 * performs erosion on the binary PGM image, also producing a
 * binary PGM image as output.  uses either a box or disc as the
 * structuring element, where size is the length of a side of the
 * box or the diameter of the disc
 */

#include "port.h"

#include <stdio.h>
#include <io.h>
#include <fcntl.h>

void write_pgm( FILE* f, int width, int height, unsigned char* data );
unsigned char* read_pgm( char* filename, int* width, int* height );

#define I(x,y)   (input_image[(y)*(width)+(x)])
#define O(x,y)   (output_image[(y)*(width)+(x)])

main( int argc, char **argv )
{
     int iwidth, iheight;
     unsigned char* input_image;
	 int sel_type;
     int sel_size;
	 unsigned char* sel;
	 unsigned char* output_image;
     
	_setmode( _fileno( stdin ), _O_BINARY );
	_setmode( _fileno( stdout ), _O_BINARY );

     if ( argc != 4 )
     {
	  fprintf( stderr, "usage: %s b|d size imagefile\n", argv[0] );
	  exit( 1 );
     }

	 switch ( argv[1][0] )
	 {
	    case 'b':
	    case 'B': sel_type = 1; break;
	    case 'd':
	    case 'D': sel_type = 2; break;
	    default: fprintf( stderr, "b|d argument must be b or d.\n" ); exit( 1 );
	  }

     sel_size = atoi( argv[2] );
     
     input_image = read_pgm( argv[3], &iwidth, &iheight );
     if ( input_image == NULL )
     {
	  fprintf( stderr, "error reading image file.\n" );
	  exit( 1 );
     }

	 sel = (unsigned char*)malloc ( sel_size * sel_size );
	 output_image = (unsigned char*)malloc ( iwidth * iheight );
     
	 // write your code here.  you should perform erosion on
	 // input_image and put the result in output_image.  use a
	 // structuring element of size sel_size.  if sel_type is 1,
	 // use a box.  if sel_type is 2, use a disc.  creating the
	 // structuring element is the first thing you should do.

     write_pgm( stdout, iwidth, iheight, output_image );
}

unsigned char* read_pgm( char* filename, int* width, int* height )
{
     char buffer[80];
     int phase = 0;
     int type, maxval;
     int j;
     unsigned char* data;
     FILE* f;

     if ( strcmp( filename, "-" ) == 0 )
     {
	  f = stdin;
     }
     else
     {
	  f = fopen( filename, "rb" );
	  if ( f == NULL )
	       return NULL;
     }
     
     while ( phase < 4 )
     {
	  fgets( buffer, 80, f );
	  if ( buffer[0] == '#' ) continue;
	  switch( phase )
	  {
	     case 0: j = sscanf( buffer, "P%d %d %d %d\n", &type, width, height, &maxval ); break;
	     case 1: j = sscanf( buffer, "%d %d %d\n", width, height, &maxval ); break;
	     case 2: j = sscanf( buffer, "%d %d\n", height, &maxval ); break;
	     case 3: j = sscanf( buffer, "%d\n", &maxval ); break;
	     case 4: j = 0; break;
	  }
	  phase += j;
     }

     if ( type != 5 )
     {
	  fprintf( stderr, "bad input image format.\n" );
	  return NULL;
     }

     data = (unsigned char*)malloc ( *width * *height );
     fread( data, *width * *height, 1, f );

     fclose( f );
     
     return data;
}

void write_pgm( FILE* f, int width, int height, unsigned char* data )
{
     fprintf( f, "P5\n%d %d\n%d\n", width, height, 255 );
     fwrite( data, width*height, 1, f );
     fclose( f );
}
